local frame = CreateFrame("Frame")

frame:RegisterEvent("ADDON_LOADED")
frame:RegisterEvent("GARRISON_MISSION_FINISHED")
frame:RegisterEvent("GARRISON_MISSION_COMPLETE")
SetCVar("lastGarrisonMissionTutorial", 8);

function init(self, event, arg1, ...)
  
  if event == "ADDON_LOADED" and arg1 == "Blizzard_GarrisonUI" then
    
    Orig_Initialize = GarrisonMissionComplete_BeginAnims;
    GarrisonMissionComplete_BeginAnims = function(...)
      --[===[@debug@
      print("GarrisonMissionComplete_BeginAnims triggered!")
      --@end-debug@]===]
      local self = ...
      self.BonusRewards.ChestModel.OpenAnim:Stop();
      self.BonusRewards.ChestModel.LockBurstAnim:Stop();
      self.BonusRewards.ChestModel:SetAlpha(1);
      for i = 1, #self.BonusRewards.Rewards do
        self.BonusRewards.Rewards[i]:Hide();
      end
      self.BonusRewards.ChestModel.LockBurstAnim:Stop();
      self.ChanceFrame.SuccessAnim:Stop();
      self.ChanceFrame.FailureAnim:Stop();
      self.BonusRewards.Saturated:Show();
      self.BonusRewards.ChestModel.Lock:Hide();
      self.BonusRewards.ChestModel:SetAnimation(0, 0);
      self.BonusRewards.ChestModel.ClickFrame:Show();
      self.ChanceFrame.ChanceText:SetAlpha(0);
      self.ChanceFrame.FailureText:SetAlpha(0);
      self.ChanceFrame.SuccessText:SetAlpha(1);
      self.ChanceFrame.Banner:SetAlpha(1);
      self.ChanceFrame.Banner:SetWidth(GARRISON_MISSION_COMPLETE_BANNER_WIDTH);
      
      self.encounterIndex = 1;
      self.animIndex = animIndex or 0;
      self.animTimeLeft = 0;
      self:SetScript("OnUpdate", GarrisonMissionComplete_OnUpdate);
      self.NextMissionButton:Enable()
      -- C_Garrison.MissionBonusRoll(GarrisonMissionFrame.MissionComplete.currentMission.missionID);
      GarrisonMissionFrame.MissionComplete.NextMissionButton:Enable();
      if ( C_Garrison.CanOpenMissionChest(GarrisonMissionFrame.MissionComplete.currentMission.missionID) ) then
        GarrisonMissionFrame.MissionComplete.BonusRewards.ChestModel:Hide();
     
        local bonusRewards = GarrisonMissionFrame.MissionComplete.BonusRewards;
        bonusRewards.waitForEvent = true;
        bonusRewards.waitForTimer = true;
        bonusRewards.success = false;
        bonusRewards:RegisterEvent("GARRISON_MISSION_BONUS_ROLL_COMPLETE");
        bonusRewards.ChestModel.OpenAnim:Play();
        C_Timer.After(0.1, GarrisonMissionComplete_OnRewardTimer);
        C_Garrison.MissionBonusRoll(GarrisonMissionFrame.MissionComplete.currentMission.missionID);
        PlaySound("UI_Garrison_CommandTable_ChestUnlock_Gold_Success");
        GarrisonMissionFrame.MissionComplete.NextMissionButton:Disable();
      end
    end
    
    GARRISON_ANIMATION_LENGTH = 0;
    
    local Or_GarrisonMissionComplete_AnimLine = GarrisonMissionComplete_AnimLine;
    GarrisonMissionComplete_AnimLine = function(...)
      --[===[@debug@
      print("GarrisonMissionComplete_AnimLine triggered!")
      --@end-debug@]===]
      GarrisonMissionFrame.MissionComplete.NextMissionButton:Enable()
      GarrisonMissionComplete_PreloadEncounterModels(self);
      
      local iconWidth = 64;           -- size of portrait icon
      local iconEdgeSpace = 6;        -- empty space on either side of the portrait
      local lineAnimDuration = 0;  -- total time for line to animate from one end to another
      local modelWaitTime = 0;        -- extra time to wait for models to load
      
      local encounters = self.Stage.Encounters;
      local lineStart = encounters.EncounterBarFill:GetLeft();
      local lineEnd = encounters["Encounter"..self.encounterIndex]:GetLeft() + iconEdgeSpace;
      local duration = lineAnimDuration / #self.animInfo;
      entry.duration = duration;
      local barFill = encounters.EncounterBarFill;
      barFill.finalWidth = lineEnd - lineStart;
      barFill:SetWidth(barFill:GetWidth() + iconWidth - iconEdgeSpace * 2);
      local barSpark = encounters.EncounterBarSpark;
      barSpark.Anim.Translation:SetDuration(duration);
      barSpark.Anim.Translation:SetOffset(barFill.finalWidth - barFill:GetWidth(), 0);
      barSpark:Show();
    end
    
    local ANIMATION_CONTROL = {
      [1] = { duration = nil,   onStartFunc = GarrisonMissionComplete_AnimLine },         -- line between encounters
      [2] = { duration = nil,   onStartFunc = GarrisonMissionComplete_AnimCheckModels },      -- check that models are loaded
      [3] = { duration = nil,   onStartFunc = GarrisonMissionComplete_AnimModels },         -- model fight
      [4] = { duration = nil,   onStartFunc = GarrisonMissionComplete_AnimPlayImpactSound },    -- impact sound when follower hits
      [5] = { duration = 0,  onStartFunc = GarrisonMissionComplete_JustReturn },       -- X over portrait
      [6] = { duration = 0,   onStartFunc = GarrisonMissionComplete_JustReturn },    -- evaluate whether to do next encounter or move on
      [7] = { duration = 0,  onStartFunc = GarrisonMissionComplete_AnimRewards },        -- reward panel
      [8] = { duration = 0,   onStartFunc = GarrisonMissionComplete_AnimLockBurst },        -- explode the lock if mission successful
      [9] = { duration = 0,   onStartFunc = GarrisonMissionComplete_AnimFollowersIn },      -- show all the mission followers
      [10] = { duration = 0,    onStartFunc = GarrisonMissionComplete_AnimXP },           -- follower xp
    }
    
    local Or_GarrisonMissionComplete_FindAnimIndexFor = GarrisonMissionComplete_FindAnimIndexFor;
    GarrisonMissionComplete_FindAnimIndexFor = function(func)
      --[===[@debug@
      print("GarrisonMissionComplete_FindAnimIndexFor triggered: ")
      --@end-debug@]===]
      for i = 1, #ANIMATION_CONTROL do
        if ( ANIMATION_CONTROL[i].onStartFunc == func ) then
          return i;
        end
      end
      return 0;
    end
    
    function GarrisonMissionComplete_JustReturn(...)
        return nil;
    end

  end
end

frame:SetScript("OnEvent", init)